/*
 * @(#)Floor3D.java    Pzt 10 May 2010 14:32:03 EEST
 *
 * 3d Object for 3d world
 */
/*
*** This program wrote under GNU/General Public License ***

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.
*/
package net.dilmerkezi.object;

import java.awt.Color;

//custom
import net.dilmerkezi.point.*;
import net.dilmerkezi.ray.*;


/**
 * @author Murat iNAN, dilmerkezi.net/eng.html
 */
public class Floor3D
     extends Object
  implements Object3D
{

    /** Colors. At least 6 colors required*/
    private final Color [] cols =
    {
        Color.yellow, Color.blue, Color.red,
        Color.green, Color.orange, Color.black
    };
    
    /** Moduler count*/
    private int fmodules = 4;
    
    /** Section value*/
    private double section;
    
    /** First constructor*/
    public Floor3D ()
    {
        section = -1.0;
    }
    
    /**
     * Second constructor
     *
     * @param n New section value
     */
    public Floor3D (double n)
    {
        this.section = n;
        fmodules = 2;
    }
    
    /**
     * Third constructor
     *
     * @param n New section value
     * @param md Module count
     */
    public Floor3D (double n, int md)
    {
        this.section = n;
        this.fmodules = md;
    }
    
    /** @return section val*/
    public double getSection ()
    {
        return section;
    }
        
    /** 
     * @param r Ray
     * @return If intersects, intersection value
     */
    public double rayIntersect (Ray3D r)
    { 
        Point3D origin = r.getOriginPoint3D ();
        Point3D direction = r.getDirectionPoint3D ();
        
        if (section-origin.getY ()*direction.getY () <= 0)
        {
            return -1;
        }
        
        double t = (section-origin.getY ())/(direction.getY ());
        
        if (t>=0.01) return t;
        else return -1;
    }

    /**
     * @param p Origin point
     * @return A suitable font size according to p
     */
    public double fontSize (Point3D p)
    {
        double d = (p.getZ ()*10);
        return d;
    }
    
    /**
     * How many ? If you want to prduce 2 coloured
     * objects use 2 etc. If you do not want using this
     * pass it like -1.
     *
     * @return Module count
     */
    public int getModuleCount ()
    {
        return fmodules;
    }
    
    /**
     * @param p Origin point
     * @return Normalized point
     */
    public Point3D normal (Point3D p)
    {
        return new Vector3D (0.0, 1.0, 0.0);
    }
    
    /**
     * @param p Point of light
     * @param r Used ray
     * @return Reflected 3D ray
     */
    public Ray3D reflect (Point3D p, Ray3D r)
    {
        double c1 = (r.getDirectionPoint3D ()).getX ();
        double c2 = -(r.getDirectionPoint3D ()).getY ();
        double c3 = (r.getDirectionPoint3D ()).getZ ();
        
        return new Ray3D (p, new Vector3D (c1, c2, c3));
    }
    
    /**
     * @param p Point 3D
     * @return A suitable color value for bg
     */
    public Color background (Point3D p)
    {
        int roundx = (int)(Math.round (p.getX ()));
        int roundz = (int)(Math.round (p.getZ ()));
        int module = (roundx+roundz)%getModuleCount ();
        
        double d = ((p.getX ()*p.getX ())+
                    (p.getY ()*p.getY ())+
                    (p.getZ ()*p.getZ ()));
        if (d < 1) return cols [0];
        
        if (module <0 ) module = 0;
        if (module >5) module = 5;
        
        return cols [module];
    }
    
    /**
     * @param p Point 3D
     * @return A suitable color value for bg
     */
    public Color foreground (Point3D p)
    {
        int roundy = (int)(Math.round (p.getY ()));
        int roundz = (int)(Math.round (p.getZ ()));
        int module = (roundy+roundz)%getModuleCount ();
        
        double d = ((p.getX ()*p.getX ())+
                    (p.getY ()*p.getY ())+
                    (p.getZ ()*p.getZ ()));
        if (d < 1) return cols [1];
        
        if (module <0 ) module = 0;
        if (module >5) module = 5;
        
        return cols [module];
    }

    /**
     * @param f Comparation floor3d
     * @return If equals true, otherwise false;
     */
    public boolean equals (Floor3D f)
    {
        return (section == f.getSection () &&
                getModuleCount () == f.getModuleCount ());
    }

    /** @return hashcode*/
    public int hashCode ()
    {
        String s = toString ();
        int result = 0;
        char [] ch = s.toCharArray ();
        int chlen = ch.length;
        int ff = 0xff;
        for (int i=0; i<chlen; i++)
        {
            result += ( ((int)(ch [i]))*ff );
        }
        return result;
    }

    /** @return Description*/
    public String toString ()
    {
        StringBuffer sb = new StringBuffer ();
        sb.append (Double.toString (section));
        sb.append ("\n---------\n");
        sb.append (Integer.toString (getModuleCount ()));
        return sb.toString ();
    }

}
/****************/
